import React from "react";
import Animated from "react-native-reanimated";
import { SpringDriver } from "@tleef/react-native-reanimated-utils/lib";
import withStage from "./withStage";
import SceneTransition from "./SceneTransition";
const { Value, SpringUtils, set, cond, block, eq, not, and, clockRunning, call } = Animated;
class SceneTransitioner extends React.Component {
    constructor(props) {
        super(props);
        this.transition = ({ from, to, springConfig }) => {
            const { stage } = this.props;
            const fromScene = stage.getScene(from);
            const toScene = stage.getScene(to);
            if (!fromScene) {
                stage.onAddScene(from, this._handleAddScene);
            }
            if (!toScene) {
                stage.onAddScene(to, this._handleAddScene);
            }
            if (from !== this.state.from || to !== this.state.to) {
                this._resetAnimation(springConfig);
            }
            this._inOut.setValue(1);
            this.setState({
                from,
                fromScene,
                to,
                toScene
            });
            return new TransitionReceipt(this._inOut, this._enterStartListeners, this._enterEndListeners, this._leaveStartListeners, this._leaveEndListeners);
        };
        this.reset = () => {
            this._resetAnimation();
            this._resetState();
        };
        this._resetAnimation = (springConfig = {}) => {
            const defaultSpringConfig = SpringUtils.makeDefaultConfig();
            this._driver = new SpringDriver({
                stiffness: springConfig.stiffness || defaultSpringConfig.stiffness,
                mass: springConfig.mass || defaultSpringConfig.mass,
                damping: springConfig.damping || defaultSpringConfig.damping,
                overshootClamping: springConfig.overshootClamping || defaultSpringConfig.overshootClamping,
                restDisplacementThreshold: springConfig.restDisplacementThreshold ||
                    defaultSpringConfig.restDisplacementThreshold,
                restSpeedThreshold: springConfig.restSpeedThreshold ||
                    defaultSpringConfig.restSpeedThreshold
            });
            this._inOut = new Value(0);
            this._prevInOut = new Value(0);
            const handleEnterEnd = cond(eq(this._prevInOut, 1), cond(not(eq(this._inOut, 1)), call([true], this._onEnterEnd), cond(not(clockRunning(this._driver.clock)), call([false], this._onEnterEnd))));
            const handleLeaveEnd = cond(eq(this._prevInOut, -1), cond(not(eq(this._inOut, -1)), call([true], this._onLeaveEnd), cond(not(clockRunning(this._driver.clock)), call([false], this._onLeaveEnd))));
            const handleEnterStart = cond(and(eq(this._inOut, 1), not(eq(this._prevInOut, 1))), call([], this._onEnterStart));
            const handleLeaveStart = cond(and(eq(this._inOut, -1), not(eq(this._prevInOut, -1))), call([], this._onLeaveStart));
            const handleEnter = cond(eq(this._inOut, 1), [
                set(this._prevInOut, 1),
                this._driver.run()
            ]);
            const handleLeave = cond(eq(this._inOut, -1), [
                set(this._prevInOut, -1),
                this._driver.rev()
            ]);
            this._drive = block([
                handleEnterEnd,
                handleLeaveEnd,
                handleEnterStart,
                handleLeaveStart,
                handleEnter,
                handleLeave
            ]);
            this._enterStartListeners = [];
            this._enterEndListeners = [];
            this._leaveStartListeners = [];
            this._leaveEndListeners = [];
        };
        this._resetState = () => {
            this.setState({
                from: undefined,
                fromScene: undefined,
                to: undefined,
                toScene: undefined
            });
        };
        this._onEnterStart = () => {
            this._enterStartListeners.forEach(fn => fn());
        };
        this._onEnterEnd = ([cancelled]) => {
            if (!cancelled) {
                this.reset();
            }
            this._enterEndListeners.forEach(fn => fn(cancelled));
        };
        this._onLeaveStart = () => {
            this._leaveStartListeners.forEach(fn => fn());
        };
        this._onLeaveEnd = ([cancelled]) => {
            if (!cancelled) {
                this.reset();
            }
            this._leaveEndListeners.forEach(fn => fn(cancelled));
        };
        this._handleAddScene = (sceneId, scene) => {
            const { from, to } = this.state;
            if (from === sceneId) {
                this.setState({
                    fromScene: scene
                });
            }
            else if (to === sceneId) {
                this.setState({
                    toScene: scene
                });
            }
            this.props.stage.removeAddSceneListener(sceneId, this._handleAddScene);
        };
        this.state = {};
        this._resetAnimation();
    }
    render() {
        const { fromScene, toScene } = this.state;
        if (!fromScene || !toScene) {
            return null;
        }
        return (<React.Fragment>
        <Animated.Code>{() => this._drive}</Animated.Code>
        <SceneTransition from={fromScene} to={toScene} position={this._driver.value}/>
      </React.Fragment>);
    }
}
class TransitionReceipt {
    constructor(inOut, enterStartListeners, enterEndListeners, leaveStartListeners, leaveEndListeners) {
        this.cancel = () => {
            this._inOut.setValue(-1);
        };
        this.onEnterStart = (handler) => {
            this._enterStartListeners.push(handler);
        };
        this.onEnterEnd = (handler) => {
            this._enterEndListeners.push(handler);
        };
        this.onLeaveStart = (handler) => {
            this._leaveStartListeners.push(handler);
        };
        this.onLeaveEnd = (handler) => {
            this._leaveEndListeners.push(handler);
        };
        this._inOut = inOut;
        this._enterStartListeners = enterStartListeners;
        this._enterEndListeners = enterEndListeners;
        this._leaveStartListeners = leaveStartListeners;
        this._leaveEndListeners = leaveEndListeners;
    }
}
export default withStage(SceneTransitioner);
//# sourceMappingURL=SceneTransitioner.js.map